@@ -3,6 +3,15 @@ import overpy
 
 class QueryResult:
     def __init__(self, result: overpy.Result, railway_type: str):
+        """ Class representing the Query Result
+
+        Parameters
+        ----------
+        result: overpy.Result
+            Result of the overpy query
+        railway_type: str
+            Railway type that was used for this query
+        """
         self.railway_type = railway_type
         self.result: overpy.Result = result
 
